;
; player AI testbed
;
; by Matt Sephton
; 
; 10 Dec 2000 - initial version, players always at target position
; 11 Dec 2000 - added mouse control, better cursor "ball" controls, added "running" to target position
; 12 Dec 2000 - improved target finding logic, now it works!
; 13 Dec 2000 - added nearest player on click code, no key controls any more - they were rubbish 
;

Const version$ = "12 Dec 2000"

Const screen_width = 800
Const screen_height = 600

Graphics screen_width,screen_height

;Const CURSOR_U=200						; UpArrow on arrow keypad
;Const CURSOR_L=203						; LeftArrow on arrow keypad
;Const CURSOR_R=205						; RightArrow on arrow keypad
;Const CURSOR_D=208						; DownArrow on arrow keypad
;Const MINUS=12							; -
;Const EQUALS=13						; +

Type playertype
	Field hx,hy		;home
	Field x#,y#		;current
	Field tx#,ty#	;target
	Field dx#,dy#	;to move
	Field vision	;vision
End Type

Type balltype
	Field x#,y#		;current
	Field dx#,dy#	;to move
End Type

Global ball.balltype=New balltype		; Create the ball.
ball\x = 400
ball\y = 300

MoveMouse ball\x,ball\y

Global team = 12-1						;number of players (0..team)
Global scale = 10
Global dist = 0
Global margin = 5
Global nearest_dist = 999
Global nearest_player = 0

Dim player.playertype(team)

Gosub init_team			;create players

SetBuffer BackBuffer()

While Not KeyDown(1)	;esc to quit
	Cls

	Gosub player_ai
	Gosub poll_controls
	Gosub draw_players
	Gosub draw_ball
	
	Gosub normal_text
	;Gosub debug_text
	
	While KeyDown(57)	;space to pause
	Wend
	;Delay 25
	
	Flip
Wend

End

;procedures
.init_team
	For t=0 To team
		player.playertype(t)=New playertype
		
		;player(t)\hx=Rnd(screen_width)
		;player(t)\hy=Rnd(screen_height)
		player(t)\hx=100+t*200 Mod screen_width
		player(t)\hy=70+t*200 Mod screen_height
		
		player(t)\x=player(t)\hx
		player(t)\y=player(t)\hy
	
		;player(t)\vision=80
		player(t)\vision=Rnd(50,100)
	Next
Return

.normal_text
	Color 255,255,255	;white
	Locate 1,1
	Print "PLAYER AI TEST by Matt Sephton (" + version + ")"
	Print "mouse moves red ball left button attracts nearest player"
Return

.debug_text
	Color 255,255,255	;white
	For t = 0 To 0	;team
		Locate 1,36
		;Print player(t)\x
		;Print player(t)\y
		Print player(t)\dx
		Print player(t)\dy
		;Print player(t)\hx + player(t)\tx
		;Print player(t)\hy + player(t)\ty
		;Print snap_dist
		;Print player(t)\vision
		Print Sgn(dist_x)
		Print Sgn(dist_y)
	Next
Return

.player_ai
	nearest_dist = 999
	For t = 0 To team
		;calculate mid point
		player(t)\tx = (ball\x - player(t)\hx) /2
		player(t)\ty = (ball\y - player(t)\hy) /2

		;simple trig. pythagoras' theorem
		dist_x = ball\x - player(t)\x
		dist_y = ball\y - player(t)\y
		dist = Sqr(dist_x^2 + dist_y^2)		;straight line dist from player to ball

		;nearest to ball
		If (dist < nearest_dist)
			nearest_dist = dist		;nearest dist to ball
			nearest_player = t		;nearest player to ball
		End If

		;avoid dbz error
		If (dist = 0) dist = 1

		;scale x and y distances to move
		player(t)\dx = (dist_x * scale) / dist
		player(t)\dy = (dist_y * scale) / dist

		;have we reached our target point? *BETTER*
		If ( Int(player(t)\hx + player(t)\tx >= player(t)\x-margin) And Int(player(t)\hx + player(t)\tx =< player(t)\x+margin) )
			player(t)\dx = 0
		Else If ( Sgn(dist_x) >= 0 And player(t)\x >= player(t)\hx + player(t)\tx )
			player(t)\dx = -player(t)\dx
		Else If ( Sgn(dist_x) < 0 And player(t)\x < player(t)\hx + player(t)\tx )
			player(t)\dx = -player(t)\dx
		End If
		If ( Int(player(t)\hy + player(t)\ty > player(t)\y-margin) And Int(player(t)\hy + player(t)\ty < player(t)\y+margin) )
			player(t)\dy = 0
		Else If ( Sgn(dist_y) >= 0 And player(t)\y >= player(t)\hy + player(t)\ty )
			player(t)\dy = -player(t)\dy
		Else If ( Sgn(dist_y) < 0 And player(t)\y < player(t)\hy + player(t)\ty )
			player(t)\dy = -player(t)\dy
		End If

		;actually do the move towards target
		player(t)\x = player(t)\x + player(t)\dx
		player(t)\y = player(t)\y + player(t)\dy

		;snap plyer to ball when it is in his vision
		;snap_x = (ball\x - player(t)\x)
		;snap_y = (ball\y - player(t)\y)
		;snap_dist = Sqr(snap_x^2 + snap_y^2)	;trig for snap distance

		;If (snap_dist <= player(t)\vision /2)
		;	player(t)\x = ball\x + Rnd(-4,4)
		;	player(t)\y = ball\y + Rnd(-4,4)
		;End If
	Next

	;on click move nearest player to ball
	If (MouseDown(1))
		;snap plyer to ball when it is in his vision
		nearest_x = (ball\x - player(nearest_player)\x)
		nearest_y = (ball\y - player(nearest_player)\y)
		nearest_dist = Sqr(nearest_x^2 + nearest_y^2)	;trig for snap distance

		;avoid dbz error
		If (nearest_dist = 0) nearest_dist = 1

		;delete (negate) nearest player movement towards ball
		player(nearest_player)\x = player(nearest_player)\x - player(nearest_player)\dx
		player(nearest_player)\y = player(nearest_player)\y - player(nearest_player)\dy

		;new scaled x and y distances to move
		player(nearest_player)\dx = (nearest_x * scale) / nearest_dist
		player(nearest_player)\dy = (nearest_y * scale) / nearest_dist

		;move nearest player towards ball
		player(nearest_player)\x = player(nearest_player)\x + player(nearest_player)\dx
		player(nearest_player)\y = player(nearest_player)\y + player(nearest_player)\dy

		;player(nearest_player)\x = ball\x + Rnd(-4,4)
		;player(nearest_player)\y = ball\y + Rnd(-4,4)
	End If
Return

.poll_controls
	;If KeyDown(CURSOR_U) ball\dy = ball\dy-0.2
	;If KeyDown(CURSOR_L) ball\dx = ball\dx-0.2
	;If KeyDown(CURSOR_R) ball\dx = ball\dx+0.2
	;If KeyDown(CURSOR_D) ball\dy = ball\dy+0.2
	
	;ball\x = ball\x + ball\dx
	;ball\y = ball\y + ball\dy

	;If (MouseDown(1))
		ball\x = MouseX(): ball\y=MouseY()
	;	ball\dx=0:ball\dy=0
	;Else
	;	MoveMouse ball\x, ball\y
	;End If
Return

.draw_ball
	Color 255,0,0		;red
	Oval ball\x-2,ball\y-2, 4,4,True	;ball
	;Oval ball\x-5,ball\y-5, 10,10,False	;ring
	Text ball\x, ball\y, "BALL"
Return

.draw_players
	For t=0 To team
		Color 127,127,127	;silver
		Oval player(t)\hx-2, player(t)\hy-2, 4,4,True
		Text player(t)\hx, player(t)\hy, Str(t)

		Color 0,0,255		;blue
		Oval player(t)\hx+player(t)\tx-2, player(t)\hy+player(t)\ty-2, 4,4,True
		Text player(t)\hx+player(t)\tx, player(t)\hy+player(t)\ty, Str(t)

		Color 0,255,0		;green
		Oval player(t)\x-(player(t)\vision/2), player(t)\y-(player(t)\vision/2), player(t)\vision,player(t)\vision,False
		Oval player(t)\x-2, player(t)\y-2, 4,4,True
		Text player(t)\x, player(t)\y, Str(t)
	Next
Return

;eof